from sift_dev_logger import getLogger
from sift_dev_logger.config import SiftDevConfig, configure
import time
import requests
import random

# Configure the logger
config = SiftDevConfig(
    service_name="python-app",
    service_instance_id="python-app-1",
    otlp_endpoint="http://88d1ef6c-3709-48b9-9441-399b323050f8.app.trysift.dev:5301",
    sift_dev_ingest_key="your-sift-dev-ingest-key",
)
configure(config)

logger = getLogger("demo.app")

def simulate_work(duration_ms=None):
    """Simulate some work being done."""
    if duration_ms is None:
        duration_ms = random.randint(100, 1000)
    time.sleep(duration_ms / 1000)
    return duration_ms

def process_data(data):
    """Example function that processes data and logs its activity."""
    logger.info("Starting data processing", extra={"data_size": len(str(data))})
    
    duration_ms = simulate_work()
    
    try:
        if isinstance(data, dict):
            result = {k.upper(): v.upper() if isinstance(v, str) else v 
                     for k, v in data.items()}
        else:
            result = str(data).upper()
            
        logger.info("Data processing completed", 
                   extra={
                       "duration_ms": duration_ms,
                       "input_type": type(data).__name__,
                       "output_type": type(result).__name__
                   })
        return result
    
    except Exception as e:
        logger.error("Error processing data", 
                    extra={
                        "error": str(e),
                        "data_type": type(data).__name__
                    })
        raise

def make_external_request(url):
    """Example function that makes HTTP requests and logs them."""
    logger.info(f"Making request to {url}")
    start_time = time.time()
    
    try:
        response = requests.get(url)
        duration_ms = (time.time() - start_time) * 1000
        
        logger.info("External request completed",
                   extra={
                       "url": url,
                       "status_code": response.status_code,
                       "duration_ms": duration_ms,
                       "response_size": len(response.content),
                       "headers": dict(response.headers)
                   })
        
        return response
        
    except Exception as e:
        logger.error("External request failed",
                    extra={
                        "url": url,
                        "error": str(e),
                        "duration_ms": (time.time() - start_time) * 1000
                    })
        raise

if __name__ == "__main__":
    # Demo different logging scenarios
    logger.info("Starting demo application")
    
    # Process different types of data
    try:
        process_data({"name": "john", "age": 30})
        process_data("Hello, World!")
        process_data(123)  # This should work
        process_data(None)  # This might cause an error
    except Exception as e:
        logger.error("Demo encountered an error", extra={"error": str(e)})
    
    # Make some external requests
    urls = [
        "https://httpbin.org/get",
        "https://httpbin.org/status/404",  # This will return 404
        "https://httpbin.org/delay/2",     # This will delay 2 seconds
    ]
    
    for url in urls:
        try:
            make_external_request(url)
        except Exception as e:
            logger.error(f"Failed to fetch {url}", extra={"error": str(e)})
    
    logger.info("Demo application completed") 