from fastapi import FastAPI, Request
from fastapi.responses import JSONResponse
from fastapi.testclient import TestClient
from sift_dev_logger.fastapi import instrument_logging_middleware
from sift_dev_logger import SiftDevConfig, configure
import pytest
import logging
import json
import time

@pytest.fixture
def app():
    app = FastAPI()
    
    # Add configuration just like in Flask tests
    config = SiftDevConfig(
        service_name="fastapi-app",
        service_instance_id="fastapi-app-1",
        otlp_endpoint="http://88d1ef6c-3709-48b9-9441-399b323050f8.app.trysift.dev:5301",
        sift_dev_ingest_key="your-sift-dev-ingest-key",
    )
    configure(config)
    
    # Instrument logging middleware
    instrument_logging_middleware(app, logger_name="fastapi.access")
    
    @app.get("/")
    async def index():
        return {"message": "Hello, World!"}
    
    @app.get("/error")
    async def error():
        raise ValueError("Test error")
    
    @app.post("/echo")
    async def echo(request: Request):
        body = await request.json()
        return {
            "headers": dict(request.headers),
            "json": body
        }
    
    @app.get("/slow")
    async def slow():
        time.sleep(1)  # Simulate slow response
        return {"message": "Slow response"}
    
    @app.post("/users")
    async def create_user(request: Request):
        body = await request.json()
        return JSONResponse(
            status_code=201,
            content={"id": 123, **body}
        )
    
    @app.exception_handler(ValueError)
    async def value_error_handler(request: Request, exc: ValueError):
        return JSONResponse(
            status_code=500,
            content={"error": str(exc)}
        )
    
    return app

@pytest.fixture
def client(app):
    return TestClient(app)

def test_successful_request(client, caplog):
    with caplog.at_level(logging.INFO, logger="fastapi.access"):  # Specify logger name
        response = client.get("/")
        
    assert response.status_code == 200
    assert response.json() == {"message": "Hello, World!"}
    
    # Find our middleware's log record (not httpx's internal logs)
    records = [r for r in caplog.records if r.name == "fastapi.access"]
    assert len(records) > 0
    record = records[-1]
    
    assert record.levelname == "INFO"
    assert "GET /" in record.message
    assert hasattr(record, "duration_ms")
    assert record.duration_ms > 0

def test_error_request(client, caplog):
    with caplog.at_level(logging.ERROR, logger="fastapi.access"):
        response = client.get("/error")
        
    assert response.status_code == 500
    assert response.json() == {"error": "Test error"}
    
    records = [r for r in caplog.records if r.name == "fastapi.access"]
    assert len(records) > 0
    record = records[-1]
    assert "GET /error" in record.message
    assert "Test error" in getattr(record, "error", "")

def test_post_request_with_json(client, caplog):
    test_data = {"name": "test", "value": 123}
    with caplog.at_level(logging.INFO, logger="fastapi.access"):
        response = client.post(
            "/echo",
            json=test_data,
            headers={"Custom-Header": "test"}
        )
    
    assert response.status_code == 200
    response_data = response.json()
    
    # Verify response contains our data
    assert response_data["json"] == test_data
    assert response_data["headers"]["custom-header"] == "test"  # FastAPI normalizes header names
    
    # Find our middleware's log record (not httpx's internal logs)
    records = [r for r in caplog.records if r.name == "fastapi.access"]
    assert len(records) > 0
    record = records[-1]
    
    # Normalize JSON strings before comparing
    actual_json = json.loads(record.request_body)
    expected_json = test_data
    assert actual_json == expected_json

def test_large_request_body(client, caplog):
    large_data = {"data": "x" * 1000000}  # 1MB of data
    with caplog.at_level(logging.INFO, logger="fastapi.access"):
        response = client.post("/echo", json=large_data)
    
    assert response.status_code == 200
    
    # Find our middleware's log record (not httpx's internal logs)
    records = [r for r in caplog.records if r.name == "fastapi.access"]
    assert len(records) > 0
    record = records[-1]
    
    # Verify log record has truncated body
    assert len(record.request_body) < 1000000
    assert "... (truncated)" in record.request_body

def test_slow_request_timing(client, caplog):
    with caplog.at_level(logging.INFO, logger="fastapi.access"):
        response = client.get("/slow")
    
    assert response.status_code == 200
    
    # Find our middleware's log record (not httpx's internal logs)
    records = [r for r in caplog.records if r.name == "fastapi.access"]
    assert len(records) > 0
    record = records[-1]
    
    # Check timing information
    assert record.duration_ms >= 1000  # Should be at least 1 second

def test_status_code_tracking(client, caplog):
    with caplog.at_level(logging.INFO, logger="fastapi.access"):
        response = client.post("/users", json={"name": "test"})
    
    assert response.status_code == 201
    
    # Find our middleware's log record (not httpx's internal logs)
    records = [r for r in caplog.records if r.name == "fastapi.access"]
    assert len(records) > 0
    record = records[-1]
    
    # Verify status code is logged
    assert record.status_code == 201
    assert "201" in record.message

def test_ignored_paths(client, caplog):
    app = FastAPI()
    instrument_logging_middleware(
        app,
        ignored_paths={"/health", "/metrics"},
        logger_name="fastapi.access"
    )
    
    @app.get("/health")
    async def health():
        return {"status": "ok"}
    
    test_client = TestClient(app)
    
    with caplog.at_level(logging.INFO, logger="fastapi.access"):
        response = test_client.get("/health")
    
    assert response.status_code == 200
    # Verify that the ignored path wasn't logged
    records = [r for r in caplog.records if r.name == "fastapi.access"]
    assert not any("/health" in record.message for record in records)

def test_custom_formatter(client, caplog):
    app = FastAPI()
    
    def custom_formatter(request, response, duration_ms):
        return f"CUSTOM: {request.method} took {duration_ms}ms"
    
    instrument_logging_middleware(
        app,
        middleware_formatter=custom_formatter,
        logger_name="fastapi.access"
    )
    
    @app.get("/test")
    async def test():
        return {"status": "ok"}
    
    test_client = TestClient(app)
    
    with caplog.at_level(logging.INFO, logger="fastapi.access"):
        response = test_client.get("/test")
    
    assert response.status_code == 200
    # Verify custom format was used
    records = [r for r in caplog.records if r.name == "fastapi.access"]
    assert any("CUSTOM: GET took" in record.message for record in records) 